/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 * Description:MViz data文件读取相关帮助类
 * Create: 2021-08-12
 */
#ifndef MODULES_VIZ_SAMPLE_SRC_UTILS_FILE_HELPER_H_
#define MODULES_VIZ_SAMPLE_SRC_UTILS_FILE_HELPER_H_

#include <string>
#include <climits>
#include <fstream>

#include <unistd.h>

#include <ara/core/vector.h>
#include <ara/log/logging.h>

#include <json/json.h>
#include <yaml-cpp/yaml.h>
#include <publisher.h>

#include "system_utils.h"

class FileUtils {
public:
    enum ReadFileReturnCode {
        OK,
        ERROR
    };

    /*
     * 功能：读取指定路径的文件
     * 输入参数：filePath-文件的路径，contents-读取的数据内容
     * 输出参数：无
     * 返回值：ReadFileReturnCode类型的枚举值
     */
    template <class T> static ReadFileReturnCode ReadFile(const std::string &filePath, T &contents)
    {
        ara::log::Logger &logger = CreateLogger("VIZS", "MViz Sample Project", ara::log::LogLevel::kVerbose);
        char absolutePath[PATH_MAX] {};
        auto ret = realpath(filePath.data(), absolutePath);
        if (ret == nullptr) {
            if (errno == ENAMETOOLONG) {
                logger.LogError() << filePath << " is too long.";
            }
            logger.LogError() << filePath << " is not exist.";
            logger.LogInfo() << "Run the \"scp\" command to copy the data folder "
                             << "in the viz_sample directory to " <<
                SystemUtils::RelativePathToAbsolutePath(SystemUtils::GetExecutablePath() + "/../");
            return ERROR;
        }
        std::ifstream in(absolutePath);
        if (!in.is_open()) {
            logger.LogError() << "open " << filePath << " failed.";
            return ERROR;
        }
        contents = { std::istreambuf_iterator<char>(in), std::istreambuf_iterator<char>() };
        return OK;
    }

    /*
     * 功能：读取二进制文件
     * 输入参数：filePath-文件的路径
     * 输出参数：无
     * 返回值：二进制文件的内容
     */
    static ara::core::Vector<uint8_t> ReadBinaryDataFile(const std::string &filePath);

    /*
     * 功能：读取Json文件
     * 输入参数：filePath-文件的路径
     * 输出参数：无
     * 返回值：Json的root节点
     */
    static Json::Value ReadJsonFile(const std::string &filePath);

    /*
     * 功能：读取Yaml文件
     * 输入参数：filePath-文件的路径
     * 输出参数：无
     * 返回值：Yaml的root节点
     */
    static YAML::Node ReadYamlFile(const std::string &filePath);
};

#endif /* MODULES_VIZ_SAMPLE_SRC_UTILS_FILE_HELPER_H_ */
